#ifndef __EXCEPTIONS_H__
#define __EXCEPTIONS_H__

#include <types.h>

/*
** Intel says there are 32 exceptions
*/
#define EXCEPTIONS_NB   32

/*
** Manage only 18 exceptions (0-19 except the 15th)
*/
#define EXCEPTION_MIN    0
#define EXCEPTION_MAX   19

/*
** Exceptions without error code
*/

#define DIVIDE_EXCEPTION        0
#define DEBUG_EXCEPTION         1
#define NMIINT_EXCEPTION        2
#define BREAKPOINT_EXCEPTION    3
#define OVERFLOW_EXCEPTION      4
#define BOUND_EXCEPTION         5
#define INVOPCODE_EXCEPTION     6
#define DEVNOTAVAIL_EXCEPTION   7
#define COPROCSEG_EXCEPTION     9
#define FLOATPOINT_EXCEPTION   16
#define MACHCHECK_EXCEPTION    18
#define SIMDFLOAT_EXCEPTION    19


/*
** Exceptions with error code
*/

#define DOUBLEFAULT_EXCEPTION     8
#define INVALIDTSS_EXCEPTION     10
#define SEGNOTPRESENT_EXCEPTION  11
#define STACKFAULT_EXCEPTION     12
#define GENERALPROT_EXCEPTION    13
#define PAGEFAULT_EXCEPTION      14
#define ALIGNCHECK_EXCEPTION     17


/*
** Exceptions reserved by intel
*/
#define UNDEF_15_EXCEPTION       15


/*
** Standard exception error code
*/
typedef struct standard_exception_error_code_fields
{
   uint_t   ext:1;     /* external event */
   uint_t   idt:1;     /* index from (1) IDT (0) GDT/LDT (cf. ti) */
   uint_t   ti:1;      /* index from (0) GDT or (1) LDT */
   uint_t   index:13;  /* selector index */
   uint_t   r:16;      /* reserved */

} __attribute__((packed)) standard_exception_error_code_fields_t;

/*
** PageFault exception error code
*/
typedef struct page_fault_error_code_fields
{
   uint_t      p:1;      /* 0 fault caused by nonpresent page, 1 page level protection violation */
   uint_t      wr:1;     /* access causing the fault was a read, 1 was a write */
   uint_t      us:1;     /* 0 processor was in supervisor, 1 was in user when fault occured */
   uint_t      rsvd:1;   /* reserved bits violation, 1 if some bits where set in reserved field */
   uint_t      id:1;     /* 1 insn fetch */
   uint_t      r:27;     /* reserved */

} __attribute__((packed)) page_fault_error_code_fields_t;

/*
** Versatile Exception Error Code
*/
typedef union exception_error_code
{
   uint32_t                                 raw;
   standard_exception_error_code_fields_t   std_excp;
   page_fault_error_code_fields_t           pf_excp;

} __attribute__((packed)) excp_err_code_t;

/*
** Functions
*/
struct mem_map;

void exceptions_init(struct mem_map*);
void general_protection_isr(ctx_t*);


#endif
