#ifndef __PIT_H__
#define __PIT_H__

#include <types.h>
#include <asmutils.h>

#define PIT_SPEED                  1193182
#define PIT_COUNTER_1MS            0x4aa
#define PIT_COUNTER0_PORT          0x40   /* generally used for scheduling */
#define PIT_COUNTER1_PORT          0x41   /* should not be touched, this is RAM refresh */
#define PIT_COUNTER2_PORT          0x42   /* partly used for sound speaker if configured on system controller port */
#define PIT_CONTROL_PORT           0x43

/*
** PIT counter 0
**
** mode 2
** LSB/MSB counter
** binary mode
**
** 10 ms tick
*/
#define PIT_COUNTER_0_DEFAULT_PERIOD     10                                                  /* in ms */
#define PIT_COUNTER_0_DEFAULT_VALUE      pit_counter_from_ms( PIT_COUNTER_0_DEFAULT_PERIOD )

#define PIT_COUNTER_0_DEFAULT_MODE_CMD   0x34

/*
** PIT Counter Port
*/
#define PIT_COUNTER_PORT(_sEl_)          (PIT_COUNTER0_PORT+(_sEl_))

/*
** LATCH command
*/
#define PIT_BASE_LATCH_CMD               0x4
#define PIT_LATCH_CMD(_sEl_)             ((PIT_BASE_LATCH_CMD)|(_sEl_)<<6)


#define __silent_pit_reset_counter(_sel_,_md_,_cnt_)		\
   ({								\
      out( (_md_), PIT_CONTROL_PORT );				\
      out( (_cnt_)&0xff, PIT_COUNTER_PORT((_sel_)) );		\
      out( ((_cnt_)>>8)&0xff, PIT_COUNTER_PORT((_sel_)) );	\
   })

#define pit_reset_counter(_sel_,_md_,_cnt_)   __silent_pit_reset_counter(_sel_,_md_,_cnt_)
#define pit_reset_counter_0(_cnt_)            pit_reset_counter(0,PIT_COUNTER_0_DEFAULT_MODE_CMD,(_cnt_))

/*
** Return current hardware counter
*/
#define pit_get_counter(_sel_)					     \
   ({								     \
      uint16_t cnt;						     \
      out( PIT_LATCH_CMD((_sel_)), PIT_CONTROL_PORT );		     \
      cnt  =  in(PIT_COUNTER_PORT((_sel_)));			     \
      cnt |= (in(PIT_COUNTER_PORT((_sel_))))<<8;		     \
      cnt;							     \
   })



/*
** Functions
*/
void   pit_init(uint8_t*,uint8_t);
void   pit_isr(ctx_t*);

#endif
