#ifndef __TYPES_H__
#define __TYPES_H__

#define PAGE_SIZE            4096
#define NULL                 ((void*)0)

#define align_inf(addr,n)    (((addr)>>(n))<<(n))
#define align_sup(addr,n)    ((((addr)>>(n))+1)<<(n))
#define align(addr,n)        (!((addr)&((1<<(n))-1)))

typedef unsigned char            uint8_t;
typedef unsigned short int       uint16_t;
typedef unsigned long int        uint32_t;
typedef unsigned long long int   uint64_t;

typedef char                     int8_t;
typedef short int                int16_t;
typedef long int                 int32_t;
typedef long long int            int64_t;

typedef unsigned int             uint_t;

typedef enum { FALSE=0, TRUE } bool;

typedef unsigned int size_t;

#define range(_val_,_start_,_end_) (((_val_)>=(_start_))&&((_val_)<=(_end_)))

/*
** Offset of a field from a structure
*/
#define offsetof(type,field)     ((uint32_t)(&(((type*)0)->field)))

/*
** Stacked Segment Registers
** by asm interrupt handler
*/
typedef struct sgr_context
{
   uint16_t   unused;
   uint16_t   ss;
   uint16_t   gs;
   uint16_t   fs;
   uint16_t   es;
   uint16_t   ds;

} __attribute__((packed)) sgr_ctx_t;


/*
** Stacked GPRs by "pusha"
*/
typedef struct gpr_context
{
   uint32_t   edi;
   uint32_t   esi;
   uint32_t   ebp;
   uint32_t   esp;
   uint32_t   ebx;
   uint32_t   edx;
   uint32_t   ecx;
   uint32_t   eax;

} __attribute__((packed)) gpr_ctx_t;

/*
** Stacked by cpu on irq
** raised from v86 mode
*/
typedef struct v86_cpu_context
{
   uint32_t   nr;
   uint32_t   err;

   uint32_t   eip;
   uint32_t   cs;         
   uint32_t   eflags;

   uint32_t   esp3;
   uint32_t   ss3;

   uint32_t   es;
   uint32_t   ds;
   uint32_t   fs;
   uint32_t   gs;

} __attribute__((packed)) v86_cpu_ctx_t;

/*
** context
*/
typedef struct context
{
   sgr_ctx_t      sgr;
   gpr_ctx_t      gpr;
   v86_cpu_ctx_t  cpu;

} __attribute__((packed)) ctx_t;

/*
** Used for MSR reading and writing
*/
typedef struct raw_msr_entry
{
   uint32_t   eax;
   uint32_t   edx;

} __attribute__((packed)) raw_msr_entry_t;

/*
** Used for 64 bits support under
** 32 bits execution machine
*/
typedef union raw_64_bits_entry
{
   uint64_t     raw;
   int64_t      sraw;
   struct
   {
      uint32_t  low;
      uint32_t  high;
   };

} __attribute__((packed)) raw64_t;

/*
** Good for anti-cast macro :)
*/
typedef union no_cast
{
   uint32_t  linear;

   void      *addr;
   uint8_t   *u8;
   uint16_t  *u16;
   uint32_t  *u32;

} __attribute__((packed)) no_cast_t;

#endif
