#ifndef __UART_H__
#define __UART_H__

#include <asmutils.h>
#include <types.h>

/*
** Base IO ports
*/
#define SERIAL_COM1             0x3f8
#define SERIAL_COM2             0x2f8
#define SERIAL_COM3             0x3e8
#define SERIAL_COM4             0x2e8

/* if LCR.dla = 0 */
#define SERIAL_TXRX(BASE)       (BASE)
#define SERIAL_TX(BASE)         (BASE)     /* WO */
#define SERIAL_RX(BASE)         (BASE)     /* RO */
#define SERIAL_IER(BASE)       ((BASE)+1)  /* RW */

/* if LCR.dla = 1 */
#define SERIAL_DLA_LSB(BASE)    (BASE)     /* RW */
#define SERIAL_DLA_MSB(BASE)   ((BASE)+1)  /* RW */

/* Always accessible */
#define SERIAL_IIR(BASE)       ((BASE)+2)  /* RO */
#define SERIAL_FCR(BASE)       ((BASE)+2)  /* WO */

#define SERIAL_LCR(BASE)       ((BASE)+3)  /* RW */
#define SERIAL_MCR(BASE)       ((BASE)+4)  /* RW */
#define SERIAL_LSR(BASE)       ((BASE)+5)  /* RO */
#define SERIAL_MSR(BASE)       ((BASE)+6)  /* RO */
#define SERIAL_SCR(BASE)       ((BASE)+7)  /* RW */

/*
** Line Status Register
*/
#define SERIAL_LSR_DATA_READY_BIT    0
#define SERIAL_LSR_OVERRUN_ERR_BIT   1
#define SERIAL_LSR_PARITY_ERR_BIT    2
#define SERIAL_LSR_FRAME_ERR_BIT     3
#define SERIAL_LSR_BREAK_INT_BIT     4
#define SERIAL_LSR_THRE_BIT          5
#define SERIAL_LSR_TSRE_BIT          6
#define SERIAL_LSR_ZERO_BIT          7


/*
** Functions
*/
#define ____uart_send_char(BASE,c)        out((c), SERIAL_TX((BASE)))
#define ____uart_recv_char(BASE)          in(SERIAL_TX((BASE)))

#define ____uart_can_send(BASE)		\
   ({					\
      uint8_t x;			\
      x = in(SERIAL_LSR((BASE)));	\
      (x & (1<<SERIAL_LSR_THRE_BIT));	\
   })

#define ____uart_can_recv(BASE)			\
   ({						\
      uint8_t x;				\
      x = in(SERIAL_LSR((BASE)));		\
      (x & (1<<SERIAL_LSR_DATA_READY_BIT));	\
   })

#define __uart_send_char(BASE,c)		\
   ({						\
      if(____uart_can_send((BASE)))		\
	 ____uart_send_char((BASE),(c));	\
   })

#define __uart_recv_char(BASE,c)		\
   ({						\
      if(____uart_can_recv((BASE)))		\
	 (c)=____uart_recv_char((BASE));	\
   })


/*
** Usable services
*/
#define uart_flush_recv(BASE)			\
   ({ char c;					\
      while( ____uart_can_recv((BASE)) )	\
	 c=____uart_recv_char((BASE));		\
   })

#define uart_enable_dla_registers(BASE)     out( 0x80, SERIAL_LCR((BASE)) )
#define uart_set_lsb_dla_rate(BASE,x)       out( (x),  SERIAL_DLA_LSB((BASE)) )
#define uart_set_msb_dla_rate(BASE,x)       out( (x),  SERIAL_DLA_MSB((BASE)) )
#define uart_send_char(_c_)                 __uart_send_char(SERIAL_COM1,(_c_))
#define uart_recv_char(_c_)                 __uart_recv_char(SERIAL_COM1,(_c_))

void    uart_init();
void    uart_write(char*);

#endif
