#ifndef __VIDEO_H__
#define __VIDEO_H__

#include <types.h>

/*
**
**  VGA registers addressing
**
** Refer to VGAREGS.TXT from VGADOC
**
** Some registers have index. So there
** is an index register to choose the index
** and a data register to send data for the
** selected index of the register
**
** A first out() is done on the register
** to select the index
**
** The data for this index in this register
** is then sent on the data port, which is
** generaly the next register
**
** Ex: sending 2 to register 3C4 index 4
**
**  out( 0x4, 0x3c4 )
**  out( 0x2, 0x3c5 )
**
*/


/*
** Hardware Cursor Control
*/
#define VIDEO_CRT_INDEX_REGISTER       0x3d4
#define VIDEO_CRT_DATA_REGISTER        0x3d5

#define VIDEO_CURSOR_START_REGISTER    0x0a
#define VIDEO_CURSOR_OFF               (1<<5)


/*
** Default text mode properties
*/
#define VIDEO_FRAME_BUFFER            0xb8000
#define VIDEO_DEFAULT_NB_COLUMNS      80
#define VIDEO_DEFAULT_NB_LINES        25


/*
** Structures defining text video mode
*/
typedef struct video_character
{
   uint8_t text;
   uint8_t color;
   
} video_char_t;

typedef struct video_line
{
   struct video_character column[VIDEO_DEFAULT_NB_COLUMNS];
   
} video_line_t;

typedef struct video_screen
{
   struct video_line line[VIDEO_DEFAULT_NB_LINES];
   
} video_screen_t;

/*
** Global text vide mode information
*/
typedef struct video_information
{
   uint8_t        default_fg_color;
   uint8_t        default_bg_color;
   uint8_t        current_column;
   uint8_t        current_line;
   uint8_t        init_column;
   uint8_t        init_line;
   video_screen_t *mem;

} video_info_t;

/*
** Functions
*/
void       video_init();
void       video_write(const char*);
void       video_put_char(char);
void       video_write_at(const char*, uint8_t*, uint8_t*, uint8_t, uint8_t);
uint32_t   video_get_absolute();
uint8_t    video_get_line();
uint8_t    video_get_col();
void       video_set_col(uint8_t);
void       video_del_char();
void       video_scroll();
void       video_clear();
void       video_clear_at(uint8_t);
void       video_reset_position();
void       video_del_cursor();
void       video_del_line_at(uint32_t);
void       video_set_line(uint8_t);

#endif
