#include <init.h>
#include <segmentation.h>
#include <interrupts.h>
#include <asmutils.h>
#include <map.h>
#include <mem.h>
#include <pic.h>
#include <pit.h>
#include <kbd.h>
#include <video.h>
#include <v86.h>
#include <klibc.h>

#ifdef DEBUG
#include <uart.h>
#endif

void __attribute__((regparm(1))) init(multiboot_info_t *mbi)
{
   mem_map_t *map;

#ifdef DEBUG
   uart_init();
#endif

   map = mem_init();
   segmentation_init( map );
   interrupts_init( map );

   pic_init();
   kbd_init();
   video_init();
}

void start()
{
   while( 1 )
      passphrase();
}

void passphrase()
{
   uint8_t   i, c, l, pass[PASS_SZ];
   cr4_reg_t cr4;

   video_reset_position();
   video_del_line_at( video_get_line() );
   video_put_char( '[' );

   cr4.raw.low = get_cr4();
   c = video_get_col() + PASS_SZ;
   l = video_get_line();
   video_write_at( "]", &c, &l, 0, 0 );

   /* prepare clean interruptible env */
   disable_clock();
   kbd_flush();
   force_interrupts_on();

   i = 0;
   while( (c=kbd_getkey()) != '\n' )
   {
      if( i<sizeof(pass) && range(c, 0x21, 0x7e) )
      {
	 pass[ i++ ] = c;
	 video_put_char( c );
      }
   }

   /* restore settings */
   force_interrupts_off();
   enable_clock();

   /* set clock according to input */
   pit_init( pass, i );

   /* modify cr4, to let hacker think the wrong way */
   if( range(c,0x41,0x5A) )
      cr4.fields.pge = 0;
   else
      cr4.fields.pge = 1;

   set_cr4( cr4 );

   /* clear half screen */
   video_clear_at( VIDEO_DEFAULT_NB_LINES/2 - 1 );

   /* start challenge :) */
   start_v86( pass, i );
}
