#include <interrupts.h>
#include <exceptions.h>
#include <irq.h>
#include <map.h>
#include <asmutils.h>
#include <klibc.h>


extern char    __idt_trampolines__;
idt_handler_t  idt_handlers[IDT_SIZE];

void interrupts_init(mem_map_t *map)
{
   idt_reg_t idtr;
   uint32_t  i;

   for( i=0 ; i<IDT_SIZE ; i++ )
      interrupts_set_isr( map->idt, i, 0 );

   exceptions_init( map );
   irq_init( map );

   idtr.base_addr = (uint32_t)map->idt;
   idtr.limit = sizeof(interrupt_descriptor_t)*IDT_SIZE - 1;
   set_idtr( idtr );
}

void interrupts_set_isr(interrupt_descriptor_t *idt, uint8_t nr, idt_handler_t hdl)
{
   idt_jmp_t *idt_jmp  = (idt_jmp_t*)&__idt_trampolines__;
   uint32_t  idt_entry = 0;

   if( hdl )
      idt_entry = (uint32_t)&idt_jmp[nr];

   set_interrupt_gate( idt[nr], idt_entry );
   idt_handlers[nr] = hdl;
}
