.text

.type  __idt_trampolines__,"object"
.globl __idt_trampolines__

.type  resume,"function"
.globl resume

.globl switch_context
.type  switch_context,"function"

/*
** Switch kernel stacks
** from current to next
**
** EAX = current
** EDX = next
**
** save current stack pointer
** load next    stack pointer
*/
switch_context:
	push	%ebp
	mov	%esp, (%eax)
	mov	(%edx),  %esp
	pop	%ebp
	ret

/*
** ACK the PIC
*/
ack_pic2:
	push	%eax
	movb	$0x20, %al
	outb	%al, $0xa0
	jmp	eoi_pic1

ack_pic1:
	push	%eax
	movb	$0x20, %al

eoi_pic1:
	outb	%al, $0x20
	pop	%eax


/*
** Stack looks like:
**
** +84  GS      [ if from V86  ]
** +80  FS      [ if from V86  ]
** +76  DS      [ if from V86  ]
** +72  ES      [ if from V86  ]
**
** +68  SS3     [ if from User ]
** +64  ESP3    [ if from User ]
**
** +60  EFLAGS
** +56  CS
** +52  EIP
** +48  ERR_CODE/0xffffffff
** +44  INT_NUMBER
**
** +40  EAX
** ...
*/
interrupt_mgr:
	pusha
	pushw	%ds
	pushw	%es
	pushw	%fs
	pushw	%gs
	pushw	%ss
	sub	$2, %esp

prepare_args:
	mov	44(%esp), %ebx
	mov	60(%esp), %edx
	mov	%esp, -4(%esp)
	sub	$4, %esp

is_v86:
	test	$0x20000, %edx
	jz	is_protected
	mov	%ss, %ax
	mov	%ax, %ds
	mov	%ax, %es
	mov	%ax, %fs
	mov	%ax, %gs
	call	v86_monitor
	jmp	pop_args

is_protected:
	lea	idt_handlers, %edx
	call	*(%edx,%ebx,4)

pop_args:
	add	$4, %esp
resume:
	add	$2, %esp
	popw	%ss
	popw	%gs
	popw	%fs
	popw	%es
	popw	%ds
	popa
	add	$8, %esp
	iret


/*
** IDT isr trampolines
*/
	.align 16
__idt_trampolines__:

/************************
   Exceptions [0 - 19]
************************/

/* divide error (no) */
	push	$0xffffffff
	push	$0
	jmp	interrupt_mgr
	.align	16

/* debug (no) */
	push	$0xffffffff
	push	$1
	jmp	interrupt_mgr
	.align	16

/* nmi (no ) */
	push	$0xffffffff
	push	$2
	jmp	interrupt_mgr
	.align	16

/* breakpoint (no) */
	push	$0xffffffff
	push	$3
	jmp	interrupt_mgr
	.align	16

/* overflow (no) */
	push	$0xffffffff
	push	$4
	jmp	interrupt_mgr
	.align	16

/* bound (no) */
	push	$0xffffffff
	push	$5
	jmp	interrupt_mgr
	.align	16

/* invalid opcode (no) */
	push	$0xffffffff
	push	$6
	jmp	interrupt_mgr
	.align	16

/* device not available (no) */
	push	$0xffffffff
	push	$7
	jmp	interrupt_mgr
	.align	16

/* double fault (yes) */
	push	$8
	jmp	interrupt_mgr
	.align	16

/* copro segment (no) */
	push	$0xffffffff
	push	$9
	jmp	interrupt_mgr
	.align	16

/* TSS invalid (yes) */
	push	$10
	jmp	interrupt_mgr
	.align	16

/* Segment not present (yes) */
	push	$11
	jmp	interrupt_mgr
	.align	16

/* stack segment fault (yes) */
	push	$12
	jmp	interrupt_mgr
	.align	16

/* general protection (yes) */
	push	$13
	jmp	interrupt_mgr
	.align	16

/* page fault (yes) */
	push	$14
	jmp	interrupt_mgr
	.align	16

/* intel reserved */
	iret
	.align	16, 33

/* fpu (no ) */
	push	$0xffffffff
	push	$16
	jmp	interrupt_mgr
	.align	16

/* alignment (yes) */
	push	$17
	jmp	interrupt_mgr
	.align	16

/* machine check (no) */
	push	$0xffffffff
	push	$18
	jmp	interrupt_mgr
	.align	16

/* simd (no) */
	push	$0xffffffff
	push	$19
	jmp	interrupt_mgr
	.align	16

/* Intel Reserved [20 - 31] */
	.rep	12
	iret
	.align	16, 0
	.endr

/* PIC 1 */
	.irp    nr,32,33,34,35,36,37,38,39
	push	$0xffffffff
 	push	$\nr
	jmp	ack_pic1
	.align	16, 0
	.endr

/* PIC 2 */
	.irp    nr,40,41,42,43,44,45,46,47
	push    $0xffffffff
	push	$\nr
	jmp	ack_pic2
	.align	16, 0
	.endr
