#include <segmentation.h>
#include <map.h>
#include <klibc.h>
#include <asmutils.h>
#include <tss.h>
#include <v86_hdl.h>

extern char __kernel_start__;

void segmentation_init(mem_map_t *map)
{
   gdt_reg_t             gdtr;
   task_state_segment_t  *tss;

   /* tss */
   tss = (task_state_segment_t*)&map->tss;
   tss->ss0  = K_DATA_SELECTOR;
   tss->io_map_addr = (uint16_t)offsetof( task_state_segment_t, io_bitmap[0] );

   /* tss bitmaps */
   memset( (void*)&tss->int_bitmap[0], 0xff, sizeof(tss->int_bitmap) );
   memset( (void*)&tss->io_bitmap[0], 0xff, sizeof(tss->io_bitmap)+1 );
   tss_init( tss );

   /* gdt */
   set_null_desc( map->gdt[K_NULL_SEG_IDX] );
   set_code_desc( map->gdt[K_CODE_SEG_IDX], K_PVL );
   set_data_desc( map->gdt[K_DATA_SEG_IDX], K_PVL );
   set_tss_desc(  map->gdt[K_TSS_SEG_IDX], tss );

   gdtr.base_addr = (uint32_t)map->gdt;
   gdtr.limit = sizeof(segment_descriptor_t)*GDT_SIZE - 1;
   set_gdtr( gdtr );

   /* now use it */
   load_selectors( K_CODE_SELECTOR, K_DATA_SELECTOR );
   set_tr( K_TSS_SELECTOR );
}
