#include <v86.h>
#include <map.h>
#include <klibc.h>
#include <v86_hdl.h>
#include <init.h>
#include <pit.h>
#include <asmutils.h>
#include <interrupts.h>
#include <exceptions.h>

extern char __kernel_end__;
extern idt_handler_t hdl_table[];

void start_v86(uint8_t *pass, uint8_t len)
{
   mem_map_t            *map;
   task_state_segment_t *tss;
   ctx_t                *ctx;
   uint32_t             *esp;

   map = (mem_map_t*)&__kernel_end__;
   tss = &map->tss;

   /* kernel stack on behalf of user task */
   tss->esp0 = (uint32_t)(map + 1);

   /* easy acces upon #GP to init user context */
   tss->eax = (uint32_t)pass;
   tss->ecx = (uint32_t)len;
   ctx = (ctx_t*)tss->esp0 - 1;

   /* trigger #GP */
   set_ldtr_reg( 5<<3 );

   /* prepare context switch */
   esp    = (uint32_t*)ctx;
   *--esp = (uint32_t)resume;
   *--esp = 0;

   switch_context( (uint32_t)&map->k_esp, (uint32_t)&esp );
}

void v86_monitor(ctx_t *ctx)
{
   if( ctx->cpu.nr == GENERALPROT_EXCEPTION )
      v86_gp(ctx);
   else if( ctx->cpu.nr == PIT_IRQ )
      pit_isr(ctx);
}

void v86_gp(ctx_t *ctx)
{
   uint8_t  *insn;
   uint32_t nr, up;

   insn = (uint8_t*)(ctx->cpu.eip + (ctx->cpu.cs<<4));

   switch( *insn )
   {
   case X86_IN_IMM_INSN:
   case X86_OUT_IMM_INSN:
      nr = (uint32_t)(*(insn+1)) + 256;
      up = 2;
      break;

   case X86_IN_REG_INSN:
   case X86_OUT_REG_INSN:
      nr = (ctx->gpr.edx & 0xffff) + 256;
      up = 1;
      break;

   case X86_INT_INSN:
      nr = (uint32_t)(*(insn+1));
      up = 2;
      break;

   default:
      return;
   }

   hdl_table[nr](ctx);
   ctx->cpu.eip += up;
}

void hdl_outb_1638(ctx_t *ctx)
{
   uint32_t   lost;
   mem_map_t  *map = (mem_map_t*)&__kernel_end__;

   print_out(ctx);
   switch_context( (uint32_t)&lost, (uint32_t)&map->k_esp );
}
