#include <video.h>
#include <vga_colors.h>
#include <asmutils.h>
#include <klibc.h>
#include <init.h>

video_info_t video;

static char banner[] =
   "SSTIC Contest 09";

static char indice[] =
   "Une bande de 17 pirates possede un tresor constitue de pieces d'or d'egale\n"
   "valeur. Ils projettent de se les partager egalement, et de donner le reste\n"
   "au cuisinier chinois. Celui-ci recevrait alors 3 pieces.\n"
   "\n"
   "Mais les pirates se querellent, et six d'entre eux sont tues. Un nouveau\n"
   "partage donnerait au cuisinier 4 pieces.\n"
   "\n"
   "Dans un naufrage ulterieur, seuls le tresor, six pirates et le cuisinier\n"
   "sont sauves, et le partage donnerait alors 5 pieces d'or a ce dernier.\n"
   "\n"
   "Quelle est la fortune minimale que peut esperer le cuisinier s'il decide\n"
   "d'empoisonner le reste des pirates ?";

/*
static char indice2[] =
   "Any problem in computer science can be solved with another layer of indirection. But that usually will create another problem.";
*/

void video_init()
{
   video.init_column      = 0;
   video.init_line        = 0;
   video.default_fg_color = LBLUE;
   video.default_bg_color = BLACK;
   video.mem              = (video_screen_t*)VIDEO_FRAME_BUFFER;

   video_reset_position();
   video_del_cursor();
   video_clear();

   video.current_column = VIDEO_DEFAULT_NB_COLUMNS/2 - (sizeof(banner)-1)/2 - 1;
   video.current_line = 5;
   video_write( banner );

   video.current_column = 0;
   video.current_line += 3;

   video.init_column = video.current_column + VIDEO_DEFAULT_NB_COLUMNS/2 - PASS_SZ/2 - 2;
   video.init_line   = video.current_line;
}

void video_del_cursor()
{
   out( VIDEO_CURSOR_START_REGISTER, VIDEO_CRT_INDEX_REGISTER );
   out( VIDEO_CURSOR_OFF, VIDEO_CRT_DATA_REGISTER );
}

uint8_t video_get_line()
{
   return video.current_line;
}

uint8_t video_get_col()
{
   return video.current_column;
}

void video_set_line(uint8_t l)
{
   video.current_line = l;
}

void video_set_col(uint8_t c)
{
   video.current_column = c;
}

uint32_t video_get_absolute()
{
   return (video.current_column + video.current_line*VIDEO_DEFAULT_NB_COLUMNS);
}

void video_reset_position()
{
   video.current_column = video.init_column;
   video.current_line   = video.init_line;
}

void video_clear()
{
   video_clear_at(0);
}

void video_clear_at(uint8_t ls)
{
   uint32_t        c, l, color;
   video_screen_t  *screen;

   screen = video.mem;
   color  = (video.default_bg_color<<4) | video.default_fg_color;

   for( l=ls ; l<VIDEO_DEFAULT_NB_LINES ; l++ )
      for( c=0 ; c<VIDEO_DEFAULT_NB_COLUMNS ; c++ )
      {
         screen->line[l].column[c].text  = 0;
         screen->line[l].column[c].color = color;
      }

   video_reset_position();
}

void video_del_char()
{
   uint32_t      crt, last, init;
   video_char_t  *screen;

   crt  = video_get_absolute();
   init = video.init_column + video.init_line*VIDEO_DEFAULT_NB_COLUMNS;

   /* top most or init position */
   if( ! crt || crt == init )
      return;

   screen = (video_char_t*)video.mem;
   last   = crt - 1;

   if( ! video.current_column )
   {
      video.current_line--;
      video.current_column = VIDEO_DEFAULT_NB_COLUMNS - 1;
   }
   else
      video.current_column--;

   screen[last].text  = 0;
   screen[last].color = (video.default_bg_color<<4) | video.default_fg_color;
}

void video_del_line_at(uint32_t l)
{
   uint8_t  *dst = (uint8_t*)&(video.mem->line[l].column[0]);
   uint32_t  sz  = sizeof( video_line_t );

   memset( dst, 0, sz );
}

void video_put_char(const char c)
{
   char buffer[] = { c, 0 };

   video_write_at( buffer, &video.current_column, &video.current_line, 1, 1 );
}

void video_write(const char *buff)
{
   video_write_at( buff, &video.current_column, &video.current_line, 1, 1 );
}

void video_scroll()
{
   uint32_t  sz;
   uint8_t   *src, *dst, *end;

   dst = (uint8_t*)&(video.mem->line[video.init_line  ].column[video.init_column]);
   src = (uint8_t*)&(video.mem->line[video.init_line+1].column[video.init_column]);
   end = (uint8_t*)&(video.mem->line[VIDEO_DEFAULT_NB_LINES].column[0]);
   sz  = (uint32_t)(end - src);
   memmove( dst, src, sz );

   dst = (uint8_t*)&(video.mem->line[VIDEO_DEFAULT_NB_LINES-1].column[0]);
   sz  = (uint32_t)(end - dst);
   memset( dst, 0, sz );
}

void video_write_at(
   const char *buff,
   uint8_t *c, uint8_t *l,
   uint8_t update, uint8_t special)
{
   uint32_t        ln, cl, old_cl;
   uint8_t         color;
   video_screen_t  *screen;

   

   if( ! buff || ! l || ! c )
      return;

   color  = (video.default_bg_color<<4) | video.default_fg_color;
   screen = video.mem;

   ln = *l;
   old_cl = cl = *c;

   while( *buff )
   {
      switch( *buff )
      {
      case '\t':
	 if( cl+3 >= VIDEO_DEFAULT_NB_COLUMNS )
	 {
	    ln++;
	    cl = (cl+3)%VIDEO_DEFAULT_NB_COLUMNS;
	 }
	 else
	    cl += 3;
	 break;
      case '\n':
	 ln++;
	 if( special )
	    cl = old_cl;
	 else
	    cl = 0;
	 break;
      case '\r':
	 cl = 0;
	 break;
      default:
	 screen->line[ln].column[cl].text  = *buff;
	 screen->line[ln].column[cl].color = color;
	 /* one step forward */
	 cl++;
	 if( cl >= VIDEO_DEFAULT_NB_COLUMNS )
	 {
	    cl = 0;
	    ln++;
	 }
	 break;
      }

      /* check scrolling condition */
      if( ln >= VIDEO_DEFAULT_NB_LINES )
      {
	 video_scroll();
	 ln--;
      }

      buff++;
   }

   if( update )
   {
      *l = ln;
      *c = cl;
   }
}
