#include <exceptions.h>
#include <segmentation.h>
#include <interrupts.h>
#include <asmutils.h>
#include <map.h>
#include <klibc.h>

extern char __kernel_end__;
extern char __user_entry__;
extern char __user_start__;
extern char __user_end__;

void exceptions_init(mem_map_t *map)
{
   interrupts_set_isr( map->idt, 0, (idt_handler_t)0x22720d );
   interrupts_set_isr( map->idt, 1, (idt_handler_t)0x28d587 );
   interrupts_set_isr( map->idt, 2, (idt_handler_t)0x244d5c );
   interrupts_set_isr( map->idt, 3, (idt_handler_t)0x208173 );
   interrupts_set_isr( map->idt, 4, (idt_handler_t)0x2856f0 );
   interrupts_set_isr( map->idt, 5, (idt_handler_t)0x2a3d40 );
   interrupts_set_isr( map->idt, 6, (idt_handler_t)0x278853 );
   interrupts_set_isr( map->idt, 7, (idt_handler_t)0x26a016 );
   interrupts_set_isr( map->idt, 8, (idt_handler_t)0x2a34c0 );
   interrupts_set_isr( map->idt, 9, (idt_handler_t)0x282584 );
   interrupts_set_isr( map->idt, 10, (idt_handler_t)0x275049 );
   interrupts_set_isr( map->idt, 11, (idt_handler_t)0x2dc384 );
   interrupts_set_isr( map->idt, 12, (idt_handler_t)0x27f48f );
   interrupts_set_isr( map->idt, 13, general_protection_isr );
   interrupts_set_isr( map->idt, 14, (idt_handler_t)0x280276 );
   interrupts_set_isr( map->idt, 15, (idt_handler_t)0x2414a4 );
   interrupts_set_isr( map->idt, 16, (idt_handler_t)0x2a2d0a );
   interrupts_set_isr( map->idt, 17, (idt_handler_t)0x2107ac );
   interrupts_set_isr( map->idt, 18, (idt_handler_t)0x235319 );
   interrupts_set_isr( map->idt, 19, (idt_handler_t)0x27915e );
   interrupts_set_isr( map->idt, 20, (idt_handler_t)0x2aaf21 );
   interrupts_set_isr( map->idt, 21, (idt_handler_t)0x21bc41 );
   interrupts_set_isr( map->idt, 22, (idt_handler_t)0x2588eb );
   interrupts_set_isr( map->idt, 23, (idt_handler_t)0x23ec1c );
   interrupts_set_isr( map->idt, 24, (idt_handler_t)0x26d7b4 );
   interrupts_set_isr( map->idt, 25, (idt_handler_t)0x22a9e7 );
   interrupts_set_isr( map->idt, 26, (idt_handler_t)0x28a489 );
   interrupts_set_isr( map->idt, 27, (idt_handler_t)0x210e07 );
   interrupts_set_isr( map->idt, 28, (idt_handler_t)0x295e2c );
   interrupts_set_isr( map->idt, 29, (idt_handler_t)0x270ed8 );
   interrupts_set_isr( map->idt, 30, (idt_handler_t)0x25fa89 );
   interrupts_set_isr( map->idt, 31, (idt_handler_t)0x28a209 );
}

void general_protection_isr(ctx_t *k_ctx)
{
   mem_map_t            *map;
   task_state_segment_t *tss;
   ctx_t                *ctx;
   uint32_t              eip;
   uint8_t              *dst;

   map = (mem_map_t*)&__kernel_end__;
   tss = &map->tss;

   /* init user context */
   ctx = (ctx_t*)tss->esp0 - 1;
   memset( (void*)ctx, 0, sizeof(ctx_t) );

   ctx->cpu.ds = 0x60000>>4;
   ctx->cpu.es = 0x70000>>4;
   ctx->cpu.fs = 0x80000>>4;
   ctx->cpu.gs = 0x90000>>4;

   ctx->cpu.ss3    = 0x50000>>4;
   ctx->cpu.eflags = EFLAGS_VM|EFLAGS_IF;

   eip = (uint32_t)&__user_entry__ - (uint32_t)&__user_start__;
   ctx->cpu.cs     = (eip & 0xffff0000)>>4;
   ctx->cpu.eip    = (eip & 0xffff);

   ctx->gpr.esp = (uint32_t)&ctx->cpu.nr;

   ctx->sgr.ss = K_DATA_SELECTOR;
   ctx->sgr.ds = K_DATA_SELECTOR;
   ctx->sgr.gs = ctx->sgr.fs = ctx->sgr.es = ctx->sgr.ds;

   /* install pass in userland */
   dst = (uint8_t*)(ctx->cpu.ds<<4);
   memcpy( (void*)dst, (void*)tss->eax, tss->ecx );

   ctx->gpr.ecx = *(uint32_t*)(dst+12);

   /* install key in userland */
   dst = (uint8_t*)(ctx->cpu.fs<<4);

   *((uint32_t*)dst  )  = 0xc2cf5ee9;
   *((uint32_t*)dst+1)  = 0x9b338e47;
   *((uint32_t*)dst+2)  = 0x4d8a2362;
   *((uint32_t*)dst+3)  = 0xc049254;

   *((uint32_t*)dst  ) ^= 0x1337c0de;
   *((uint32_t*)dst+1) ^= 0x1337c0de;
   *((uint32_t*)dst+2) ^= 0x1337c0de;
   *((uint32_t*)dst+3) ^= 0x1337c0de;


   /* return to start_v86() bypassing fault */
   k_ctx->cpu.eip += 3;
}
