#!/bin/sh

if [ $# -lt 2 ];then
	echo "usage: $0 <img> <kernel> [user programs]"
	exit 1
fi

GRUB="/usr/sbin/grub"
GRUB_DIR="/boot/grub"
TMP_DIR="/tmp/mnt-$$"
IMG=$1
shift
K_BIN=$1
shift
U_BIN=$@
K_BASE=$(basename $K_BIN)

mkdir -p ${TMP_DIR}

if [ ! -s ${IMG} ]; then
    dd if=/dev/zero of=${IMG} bs=512 count=2880
    mke2fs -F ${IMG}
    mount -o loop ${IMG} ${TMP_DIR}
    mkdir -p ${TMP_DIR}/grub
    cp ${GRUB_DIR}/stage1 ${TMP_DIR}/grub/
    cp ${GRUB_DIR}/stage2 ${TMP_DIR}/grub/

    # create conf
    cat <<EOT >${TMP_DIR}/grub/menu.lst
default 0
timeout 0
title kernel
root (fd0)
kernel /${K_BASE}
EOT

    # modules conf
    for module in ${U_BIN}
    do
      echo "module /$(basename ${module})" >> ${TMP_DIR}/grub/menu.lst
    done

    # create image
    ${GRUB} --batch <<EOT
device (fd0) ${IMG}
install (fd0)/grub/stage1 (fd0) (fd0)/grub/stage2 (fd0)/grub/menu.lst
EOT
    
else
    mount -o loop ${IMG} ${TMP_DIR}
fi

# anyway copy kernel and modules
cp ${K_BIN} ${U_BIN} ${TMP_DIR}/

# clean quit
umount ${TMP_DIR}
rm -rf ${TMP_DIR}




