#!/usr/bin/env python

import random, struct, sys, array, os, md5

###
### Parameters
###

FAST = 0
GCC_MODULUS = 1
USER_INT_NR = 45

# patch here with .user size:
#
# readelf -a kernel/src/gen/user.o | grep ' .user ' | cut -d' ' -f28
#
user_section_sz = 0x1200

# patch here with .code size
kernel_code_sz = 0xe0000

# compute random ranges
kernel_start_rand = 0x20000c
kernel_end_rand = kernel_start_rand+kernel_code_sz

ivt_sz = 0x400
ivt_hdl_size = user_section_sz - ivt_sz
ivt_hdl_start_rand = ivt_sz
ivt_hdl_end_rand = user_section_sz

###
### Code
###

# poor man stats
stat_cr = []
stat_cg = []
stat_jk = []
stat_fw = 0

#
# Set TSS bitmap (int and i/o)
#
def tss_allow(ar,n):
    ar[n/8] &= ~(1<<(n%8))
#endf

#
# Generate junk code
#
def gen_jk_role(tp, mn, op, sys):
    global stat_jk

    stat_jk[mn][tp] += 1

    header = "/* junk */"
    if tp == 0:
        idx1 = str(random.randint(0,2))
        idx2 = str(random.randint(0,7))
        code = "void hdl_"+str_mn_insn(mn)+"_"+str(op)+"(ctx_t *ctx)\n{\n" + \
               "   uint32_t src =\n" + \
               "        (ctx->cpu.ds<<4) +\n" + \
               "        (ctx->gpr.esi & 0xffff) +\n" + \
               "        ("+idx1+"*8*2)+("+idx2+"*2);\n" + \
               "   uint32_t dst =\n" + \
               "        (ctx->cpu.gs<<4) +\n" + \
               "        (ctx->gpr.esi & 0xffff) +\n" + \
               "        ("+idx2+"*2);\n" + \
               "   ctx->gpr.eax = (*(uint32_t*)src);\n" + \
               "   ctx->gpr.eax = ((ctx->gpr.eax>>8)&0xff) | ((ctx->gpr.eax&0xff)<<8);\n" + \
               "   ctx->gpr.eax = (ctx->gpr.eax>>24)|(ctx->gpr.eax<<24)|((ctx->gpr.eax>>8)&0xff00)|((ctx->gpr.eax<<8)&0xff0000);\n" + \
               "   ctx->gpr.eax = ((ctx->gpr.eax>>8)&0xff) | ((ctx->gpr.eax&0xff)<<8);\n" + \
               "   *(uint32_t*)dst = ctx->gpr.eax;\n" + \
               "}\n"
    else:
        idx1 = str(random.randint(0,2)*2)
        idx2 = str(random.randint(0,7)*2)
        code = "\tmovl\t$"+idx1+", %eax\n" + \
               "\tmovl\t"+idx2+"(%esi,%eax,8), %eax\n" + \
               "\trorw\t$8, %ax\n" + \
               "\tbswap\t%eax\n" + \
               "\trorw\t$8, %ax\n" + \
               "\tmovl\t%eax, %gs:"+idx2+"(%esi)"

    return [ header, code, (mn, op) ]
#endef

#
# Generate user CR code
#
def gen_cr_u_code(delta,p,idx):
    if p == 1:
        return "\tmovl\t$0x%x" % delta+", %edi\n" + \
               "\tmovl\t%edi, %ebp\n" + \
               "\tshrl\t$2, %ebp\n" + \
               "\tandl\t$3, %ebp\n"
    elif p == 2:
        return "\tmovl\t$"+str(idx+1)+", %edx\n" + \
               "\tmovl\t(%esi,%edx,4), %ebx\n" + \
               "\tdecl\t%edx\n" + \
               "\tmovl\t%edx, %eax\n" + \
               "\tandl\t$3, %eax\n" + \
               "\txorl\t%ebp, %eax\n" + \
               "\tmovl\t%fs:(%esi, %eax, 4), %eax\n" + \
               "\txorl\t%ecx, %eax\n" + \
               "\tpushl\t%edi\n" + \
               "\tpushl\t%ecx\n" + \
               "\tpushl\t%ebx\n" + \
               "\txorl\t%ebx, %edi\n" + \
               "\taddl\t%edi, %eax\n" + \
               "\tshrl\t$3, %ebx\n" + \
               "\tshll\t$4, %ecx\n" + \
               "\txorl\t%ecx, %ebx\n" + \
               "\tmovl\t%ebx, %edi\n" + \
               "\tmovl\t0(%esp), %ebx\n" + \
               "\tmovl\t4(%esp), %ecx\n" + \
               "\tshrl\t$5, %ecx\n" + \
               "\tshll\t$2, %ebx\n" + \
               "\txorl\t%ecx, %ebx\n" + \
               "\taddl\t%ebx, %edi\n" + \
               "\txorl\t%edi, %eax\n" + \
               "\tpopl\t%ebx\n" + \
               "\tpopl\t%ecx\n" + \
               "\tpopl\t%edi\n" + \
               "\tmovl\t%eax, %ecx\n" + \
               "\tmovl\t(%esi,%edx,4), %eax\n" + \
               "\taddl\t%eax, %ecx\n" + \
               "\tmovl\t%ecx, (%esi,%edx,4)\n"
    elif p == 3:
        return "\tmovl\t(%esi), %ebx\n" + \
               "\tmovl\t$"+str(idx)+", %edx\n" + \
               "\tandl\t$3, %edx\n" + \
               "\txorl\t%ebp, %edx\n" + \
               "\tmovl\t%fs:(%esi, %edx, 4), %eax\n" + \
               "\txorl\t%ecx, %eax\n" + \
               "\tpushl\t%edi\n" + \
               "\tpushl\t%ecx\n" + \
               "\tpushl\t%ebx\n" + \
               "\txorl\t%ebx, %edi\n" + \
               "\taddl\t%edi, %eax\n" + \
               "\tshrl\t$3, %ebx\n" + \
               "\tshll\t$4, %ecx\n" + \
               "\txorl\t%ecx, %ebx\n" + \
               "\tmovl\t%ebx, %edi\n" + \
               "\tmovl\t0(%esp), %ebx\n" + \
               "\tmovl\t4(%esp), %ecx\n" + \
               "\tshrl\t$5, %ecx\n" + \
               "\tshll\t$2, %ebx\n" + \
               "\txorl\t%ecx, %ebx\n" + \
               "\taddl\t%ebx, %edi\n" + \
               "\txorl\t%edi, %eax\n" + \
               "\tpopl\t%ebx\n" + \
               "\tpopl\t%ecx\n" + \
               "\tpopl\t%edi\n" + \
               "\tmovl\t%eax, %ecx\n"
    # p == 4
    return "\tmovl\t12(%esi), %eax\n" + \
           "\taddl\t%eax, %ecx\n" + \
           "\tmovl\t%ecx, 12(%esi)\n" + \
           "\txorl\t%edi, %edi"
#endef

#
# Generate kernel CR code
#
def gen_cr_k_code(delta,p,idx):
    if p == 1:
        return "   ctx->gpr.edi = 0x%x" % delta+";\n" + \
               "   ctx->gpr.ebp = (ctx->gpr.edi>>2)&3;\n"
    elif p == 2:
        return "   uint32_t mx1,mx2;\n" + \
               "   uint32_t v = (ctx->cpu.ds<<4) + (ctx->gpr.esi & 0xffff);\n" + \
               "   uint32_t k = (ctx->cpu.fs<<4) + (ctx->gpr.esi & 0xffff);\n" + \
               "   ctx->gpr.ebx = ((uint32_t*)v)["+str(idx+1)+"];\n" + \
               "   mx1 = ((((uint32_t*)k)[("+str(idx)+"&3)^ctx->gpr.ebp])^ctx->gpr.ecx) + (ctx->gpr.edi^ctx->gpr.ebx);\n" + \
               "   mx2 = ((ctx->gpr.ebx>>3)^(ctx->gpr.ecx<<4)) + ((ctx->gpr.ebx<<2)^(ctx->gpr.ecx>>5));\n" + \
               "   ((uint32_t*)v)["+str(idx)+"] += mx1^mx2;\n" + \
               "   ctx->gpr.ecx = ((uint32_t*)v)["+str(idx)+"];\n"
    elif p == 3:
        return "   uint32_t mx1,mx2;\n" + \
               "   uint32_t v = (ctx->cpu.ds<<4) + (ctx->gpr.esi & 0xffff);\n" + \
               "   uint32_t k = (ctx->cpu.fs<<4) + (ctx->gpr.esi & 0xffff);\n" + \
               "   ctx->gpr.ebx = *((uint32_t*)v);\n" + \
               "   mx1 = ((((uint32_t*)k)[("+str(idx)+"&3)^ctx->gpr.ebp])^ctx->gpr.ecx) + (ctx->gpr.edi^ctx->gpr.ebx);\n" + \
               "   mx2 = ((ctx->gpr.ebx>>3)^(ctx->gpr.ecx<<4)) + ((ctx->gpr.ebx<<2)^(ctx->gpr.ecx>>5));\n" + \
               "   ctx->gpr.ecx = mx1^mx2;\n"

    return "   uint32_t v = (ctx->cpu.ds<<4) + (ctx->gpr.esi & 0xffff);\n" + \
           "   ctx->gpr.ecx += ((uint32_t*)v)[3];\n" + \
           "   ((uint32_t*)v)[3] = ctx->gpr.ecx;\n" + \
           "   ctx->gpr.edi = 0;\n"
#endef

#
# Generate crypto code
#
def gen_cr_role(tp, mn, op, cr_sys):
    global stat_cr
    r = random.randint(0,2)

    # fallback to junk code
    if r == 0 or len(cr_sys) == 0:
        return gen_jk_role(tp, mn, op, cr_sys)

    # generate CR code
    stat_cr[mn][tp] += 1
    cr_line = cr_sys[len(cr_sys)-1]
    delta = cr_line[0]
    p = cr_line[1].pop()
    if p >= 2 :
        idx = 4 - len(cr_line[1])
    else:
        idx = 0
    if len(cr_line[1]) == 0:
        cr_sys.pop()

    header = "/* cr p"+str(p)+" delta #0x%x" % delta+" */"

    # userland
    if tp == 1 :
        code = gen_cr_u_code(delta,p,idx)
    # kernel
    else:
        code = "void hdl_"+str_mn_insn(mn)+"_"+str(op)+"_cr_p"+str(p)+"_delta_"+str(delta)+"(ctx_t *ctx)\n{\n"
        code += gen_cr_k_code(delta,p,idx)
        code += "}\n"

    return [ header, code, (mn,op) ]
#endef

#
# Generate Congurence code
#
def gen_cg_role(tp, mn, op, cg_sys):
    global stat_cg
    r = random.randint(0,2)

    # fallback to junk code
    if r == 0 or len(cg_sys) == 0:
        return gen_jk_role(tp, mn, op, cg_sys)

    # generate CG code
    stat_cg[mn][tp] += 1
    idx = random.randint(0,len(cg_sys)-1)
    s = cg_sys[idx]
    sys_idx = s[0]
    equ_idx = s[1]
    div = 1
    val = s[2].pop()

    if len(s[2]) == 0:
        div = 0
        cg_sys.remove( s )

    # div
    if div == 1:
        header = "/* cg div "+str(val)+" */"
        # userland
        if tp == 1 :
            code = "\tmovl\t$"+str(equ_idx*2)+", %ebx\n" + \
                   "\txorw\t%dx, %dx\n" + \
                   "\tmovw\t"+str(sys_idx*2)+"(%si), %ax\n" + \
                   "\tmovw\t$"+str(val)+", %cx\n" + \
                   "\tdivw\t%cx\n" + \
                   "\tmovw\t%dx, %es:"+str(sys_idx*2)+"(%edi,%ebx,8)"
        # kernel
        else:
            code = "void hdl_"+str_mn_insn(mn)+"_"+str(op)+"_div_"+str(val)+"(ctx_t *ctx)\n{\n" + \
                   "   uint32_t src =\n" + \
                   "        (ctx->cpu.ds<<4) +\n" + \
                   "        (ctx->gpr.esi & 0xffff) +\n" + \
                   "        "+str(sys_idx)+"*2;\n" + \
                   "   uint32_t dst =\n" + \
                   "        (ctx->cpu.es<<4) +\n" + \
                   "        (ctx->gpr.edi & 0xffff) +\n" + \
                   "        "+str(equ_idx)+"*8*2+"+str(sys_idx)+"*2;\n"

            if GCC_MODULUS :
                code += "   *(uint16_t*)dst = (*(uint16_t*)src)%"+str(val)+";\n}\n"
            else:
                code += "   asm volatile ( \"divw %%cx\" :\"=d\"(*(uint16_t*)dst):\"a\"(*(uint16_t*)src),\"d\"(0),\"c\"("+str(val)+") );\n}\n"
    # mod
    else:
        header = "/* cg mod "+str(val)+" */"
        # userland
        if tp == 1 :
            code = "\tmovl\t$"+str(equ_idx*2)+", %ecx\n" + \
                   "\tmovw\t%es:"+str(sys_idx*2)+"(%edi,%ecx,8), %ax\n" + \
                   "\trorw\t$8, %ax\n" + \
                   "\tnotw\t%ax\n" + \
                   "\tmovw\t%ax, %es:"+str(sys_idx*2)+"(%edi,%ecx,8)"
        # kernel
        else:
            code = "void hdl_"+str_mn_insn(mn)+"_"+str(op)+"_mod_"+str(val)+"(ctx_t *ctx)\n{\n" + \
                   "   uint16_t val;\n" + \
                   "   uint32_t dst =\n" + \
                   "        (ctx->cpu.es<<4) +\n" + \
                   "        (ctx->gpr.edi & 0xffff) +\n" + \
                   "        "+str(equ_idx)+"*8*2+"+str(sys_idx*2)+";\n" + \
                   "  val = *(uint16_t*)dst;\n" + \
                   "  val = ~( ((val&0xff)<<8) | ((val&0xff00)>>8) );\n" + \
                   "  *(uint16_t*)dst = val;\n" + \
                   "}\n"

    return [ header, code, (mn,op) ]
#endef


# generate an instruction:
#
# mn: int=0, in=1, out=2
# op: port or interrupt number 
# tp: 0 kernel, 1 user, 2 user indirect
#
def gen_insn(gen_role, system, tss, ivt, hdl, khdl, int_l, io_l, excl, depth=0):

    global stat_fw

    if len(io_l) == 0:
        if excl != 0:
            print "Raaaaa no more I/O ports in list !"
            sys.exit(-1)
        else:
            mn = 0
    elif len(int_l) == 0:
        if excl != 0:
            print "Raaaaa no more Int #NR in list !"
            sys.exit(-1)
        else:
            mn = random.randint(1,2)
    else:
        mn = random.randint(0,3)

    # reduce to int type 
    if mn > 2:
        mn = 0

    # IN
    if mn == 1:
        op = io_l.pop()
        tp = random.randint(0,1)
        if tp == 0 :
            role = gen_role( tp, mn, op, system )
            khdl.append( role )
        else:
            tss_allow( tss[1], op )
            role = None

        return (mn,op,tp,None,role)
        
    # OUT
    if mn == 2 :
        op = io_l.pop()
        tp = 0
        role = gen_role( tp, mn, op, system )
        khdl.append( role )

        return (mn,op,tp,None,role)

    # INT
    tp = random.randint(0,2)
    op = int_l.pop()
    insn = None
    role = None

    # limit depth
    if depth == 3 and tp == 2:
        tp = random.randint(0,1)

    # build user handler and ivt entry
    if tp != 0:
        tss_allow( tss[0], op )
        eip_lbl = "lbl_int_"+str(op)
        cs  = 0
        ivt[op] = "\t.word\t"+eip_lbl+"\n\t.word\t"+str(cs)
        hdl[op] = eip_lbl+":\n"
        if tp == 2:
            stat_fw += 1
            insn = gen_insn(gen_role,system,tss,ivt,hdl,khdl,int_l,io_l,excl,depth+1)
            hdl[op] += str_insn(insn)
        else:
            role = gen_role( tp, mn, op, system )
            hdl[op] += role[1]

        hdl[op] += "\n\tiret"
    else:
        role = gen_role( tp, mn, op, system )
        khdl.append( role )

    return (mn,op,tp,insn,role)
#endef

#return string for insn's mnemonic
def str_mn_insn(mn):
    if mn == 0:
        return "int"
    elif mn == 1 :
        return "inb"
    elif mn == 2:
        return "outb"
#endef

#return string for insn
def str_insn(insn):

    #get mnemonic string
    s = "\t"+str_mn_insn(insn[0])+"\t"

    #int
    if insn[0] == 0:
        s = s+"$0x%x" % insn[1]+"\t"
    #i/o
    else:
        if insn[1] < 0x100:
            if insn[0] == 1 :
                s = s+"$0x%x" % insn[1]+", %al"
            else:
                s = s+"%al, $"+"0x%x" % insn[1]
        else:
            s = "\tmovw\t$0x%x" % insn[1]+", %dx\n"+s
            if insn[0] == 1 :
                s = s+"%dx, %al"
            else:
                s = s+"%al, %dx"

    s += "\t"

    # kernel marker
    if insn[2] == 0:
        s = s+"\t/* kernel */"
    # redirection marker
    elif insn[3] != None:
        s = s+"\t/* redir "+str_mn_insn(insn[3][0])+" 0x%x" % insn[3][1]+" */"

    # role marker
    if insn[4] != None:
        s = s+"\t"+insn[4][0]

    return s
#endef

# build the passphrase
def gen_passwd(sz):
    passwd = "".join([ chr(random.randint(0x21,0x7e)) for i in range(0,sz) ])
    t = sorted(passwd)
    for i in range(0,len(t)-1):
        if t[i] == t[i+1] :
            return gen_passwd(sz)
    return passwd
#endef

def show_passwd(passwd):
    print "Passphrase:", passwd
    print map( lambda x: hex(ord(x)), passwd )
#endef

def show_cipher(passwd,key,xor_key,sys):

    print "\nCipher passphrase:"
    print map( lambda x: hex(x), passwd )
    print "\nCipher key:"
    print map( lambda x: hex(x), key )
    print "\nCipher xored key:"
    print map( lambda x: hex(x), xor_key )
    print "\nCipher (reversed) deltas:"
    print map( lambda x: hex(x[0]), sys )
    print
#endef

#
# generate congruence systems
#
# [ sys_idx, equ_idx, [m,d] ], ...
#
def pgcd(a,b):
    if a == 0:
        return b
    return abs(pgcd(b%a,a))
#endef

def gen_cg_sys_prime(max):

    y1 = random.randint(1,max)
    y2 = random.randint(1,max)
    while pgcd(y1,y2) != 1:
        y2 = random.randint(1,max)

    y3 = random.randint(1,max)
    while pgcd(y2,y3) != 1 or pgcd(y1,y3) != 1:
        y3 = random.randint(1,max)
    
    return [y1,y2,y3]
#endef

def gen_cg_sys(cr_pass):
    sys = []
    divisor = []
    print "Congruence system:"
    for i in range(0,len(cr_pass)):
        x = cr_pass[i]
        txt = str(x)+" ="
        y_l = gen_cg_sys_prime( x/2 )
        divisor.append( y_l )
        for j in range(0,3):
            y = y_l[j]
            m = x%y
            print txt,m,"mod",y
            sys.append( [i,j,[m,y]] )
    print
    return sys,divisor
#endef

#
# Compute operation on Modulus
# and show it in correct order
#
def compute_solution(system):
    table = {}
    tab = []

    # order it
    for i in range(0,24):
        table[(i%3)*8 + i/3] = system[i][2][0]

    # ror ~ modulus
    for i in range(0,24):
        table[i] = (~( ((table[i]&0xff)<<8) | ((table[i]>>8)&0xff) ))&0xffff

    print "Base result:"
    for i in range(0,24):
        print "%x" % table[i],
    print "\n"

    # make simple list of long
    for i in range(0,24,2):
        tab.append( ((table[i+1])<<16 | table[i])&0xffffffff )

    # show it
    print "Challenge answer:"
    for x in tab:
        print "%x" % x,
    print "\n"

    # generata embedded solution
    a_k = 0xa5cc1a27
    a_s = []
    print "Embedded answer:"
    for x in tab:
        v = (x^a_k)&0xffffffff
        a_s.append( v )
        print "%x" % v,
    print "\n"

    return tab,a_s,a_k
#endef

#
# Cipher passphrase
# return short arrays (clear,ciphered)
# to be used by CG system
#
# Build a system to easily inject CR code
#
#  [
#    [ delta18, [4, 3, 2, 2, 2, 1] ],
#    ...
#    [ delta0, [4, 3, 2, 2, 2, 1] ]
#  ]
#
def cr_mx(z,y,sm,p,k,e):
    return (((z>>5^y<<2)+(y>>3^z<<4))^((sm^y)+(k[(p&3)^e]^z)))
#endef

def delta_has_no_null(delta):
    if delta&0xff != 0 and (delta>>8)&0xff != 0 and (delta>>16)&0xff != 0 and (delta>>24)&0xff != 0:
        return 1
    return 0
#endef

def gen_cr_sys(passwd):

    sys = []
    for i in range(0,19):
        good_delta = 0
        delta = 0
        while not good_delta:
            delta = random.randint(0x11111111,0xffffffff)
            # exclude original XXTea constant and null bytes
            if delta != 0x9e3779b9 and delta_has_no_null(delta) :
                good_delta = 1
        sys.append( [ delta, [4,3,2,2,2,1] ] )

    k   = [ random.randint(0,0xffffffff) for i in range(0,4) ]
    v   = [ struct.unpack("<L", passwd[i:i+4])[0] for i in range(0, len(passwd), 4) ]
    z   = v[3]

    for s in sys:
        sm = s[0]
        e = (sm>>2)&3
        p = 0
        while p < 3:
            y = v[p+1]
            v[p] = (v[p] + cr_mx(z,y,sm,p,k,e))&0xffffffff
            z = v[p]
            p += 1

        y = v[0]
        v[3] = (v[3] + cr_mx(z,y,sm,p,k,e))&0xffffffff
        z = v[3]

    # we use pop() into gen_cr_role()
    sys.reverse()

    # save deltas reversed for keygen
    rdeltas = [ d[0] for d in sys ]

    # prepare xored_key to embed into kernel code
    xkv = 0x1337c0de
    xk = [ (x^xkv)&0xffffffff for x in k ]

    # build short array
    sv = []
    for x in v:
        sv.append( int(x&0xffff) )
        sv.append( int((x>>16)&0xffff) )

    return sv,k,xk,xkv,sys,rdeltas
#endf

# subset of io and int range
# to fill when system is empty
def fill_io_int(insns,tss,ivt,hdl,khdl,int_range,io_range,io_f,int_f):

    global FAST

    if FAST:
        return

    l = len(io_range)/io_f
    sub_io = []
    for i in range(0,l):
        sub_io.append(io_range[0])
        io_range.pop(0)

    l = len(int_range)/int_f
    sub_int = []
    for i in range(0,l):
        sub_int.append(int_range[0])
        int_range.pop(0)

    print "JK adding: "+str(len(sub_io))+" io and "+str(len(sub_int))+" int"
    while len(sub_io) != 0 or len(sub_int) != 0:
        insns.append(gen_insn(gen_jk_role,None,tss,ivt, hdl, khdl, sub_int, sub_io, 0) )
#endf


# generate whole code
def gen():
    global stat_jk
    global stat_cr
    global stat_cg
    global stat_fw
    global FAST

    print """
\t###############################################
\t############# Challenge Generator #############
\t###############################################
"""
    sz = 16

    # generate input
    if FAST:
        passwd = "a"*16
    else:
        passwd = gen_passwd( sz )

    show_passwd( passwd )

    # generate CR system and pass input into it
    cr_passwd, cr_key, x_cr_key, x_cr_key_val, cr_sys, r_deltas = gen_cr_sys( passwd )
    show_cipher(cr_passwd, cr_key, x_cr_key, cr_sys)

    # generate CG system with RMized input
    cg_sys, cg_divisor = gen_cg_sys( cr_passwd )

    # finish challenge
    a_std,a_xor,a_k = compute_solution( cg_sys )

    # shuffle CG operations order
    random.shuffle( cg_sys )

    # generate our INT,IN,OUT ranges
    int_range = range(1,256)
    int_range.remove(3)

    print "Removed int and IO: int0 int3 and io 0x666\n"

    io_range = range(0x100,0x10000)
    io_range.remove(0x666)

    random.shuffle( int_range )
    random.shuffle( io_range )

    insns = []
    tss   = ( array.array('B', [0xff for i in range(0,32)]),
              array.array('B', [0xff for i in range(0,8192)]) )
    hdl   = {}
    ivt   = {}
    khdl  = []

    # empty IVT and user handlers
    for i in range(0,256):
        hdl[i] = ""
        cs  = 0
        eip = random.randint(ivt_hdl_start_rand, ivt_hdl_end_rand)
        ivt[i] = "\t.word\t"+str(eip)+"\n\t.word\t"+str(cs)

    # generate CR code
    stat_cr = [[0,0],[0,0],[0,0]]
    stat_cg = [[0,0],[0,0],[0,0]]
    stat_jk = [[0,0],[0,0],[0,0]]
    stat_fw = 0
    while len(cr_sys) != 0:
        insns.append(gen_insn(gen_cr_role,cr_sys,tss,ivt,hdl,khdl,int_range,io_range,1))

    print "CR system inserted into user code !"
    print "CR stats (k,u): "+ \
          str(stat_cr[0][0])+","+str(stat_cr[0][1])+" int, " + \
          str(stat_cr[1][0])+","+str(stat_cr[1][1])+" in, " + \
          str(stat_cr[2][0])+","+str(stat_cr[2][1])+" out"
    print "JK stats (k,u): "+ \
          str(stat_jk[0][0])+","+str(stat_jk[0][1])+" int, " + \
          str(stat_jk[1][0])+","+str(stat_jk[1][1])+" in, " + \
          str(stat_jk[2][0])+","+str(stat_jk[2][1])+" out"
    print "FW stats: "+str(stat_fw)
    print "-> still: "+str(len(io_range))+" i/o and "+str(len(int_range))+" int\n"

    # generate Congruence Code
    stat_fw = 0
    stat_jk = [[0,0,0],[0,0],[0,0]]
    while len(cg_sys) != 0 :
        insns.append(gen_insn(gen_cg_role,cg_sys,tss,ivt,hdl,khdl,int_range,io_range,1))

    print "CG system inserted into user code !"
    print "CG stats (k,u): "+ \
          str(stat_cg[0][0])+","+str(stat_cg[0][1])+" int, " + \
          str(stat_cg[1][0])+","+str(stat_cg[1][1])+" in, " + \
          str(stat_cg[2][0])+","+str(stat_cg[2][1])+" out"
    print "JK stats (k,u): "+ \
          str(stat_jk[0][0])+","+str(stat_jk[0][1])+" int, " + \
          str(stat_jk[1][0])+","+str(stat_jk[1][1])+" in, " + \
          str(stat_jk[2][0])+","+str(stat_jk[2][1])+" out"
    print "FW stats: "+str(stat_fw)
    print "-> still: "+str(len(io_range))+" i/o and "+str(len(int_range))+" int\n"

    # WTF !
    if not FAST :
        if len(int_range) > USER_INT_NR :
            return None,None,None,None,None,None,(None,None),(None,None),None,None,1

    # put some random int i/o
    stat_fw = 0
    stat_jk = [[0,0,0],[0,0],[0,0]]
    io_rate = 25
    fill_io_int(insns,tss,ivt,hdl,khdl,int_range,io_range, io_rate, 2)

    # show output and leave
    insns.append( (2,0x666,0,None,None) )
    print "XX magic output inserted !"

    # put last random int i/o
    fill_io_int(insns,tss,ivt,hdl,khdl,int_range,io_range, io_rate, 1)

    print "JK stats (k,u): "+ \
          str(stat_jk[0][0])+","+str(stat_jk[0][1])+" int, " + \
          str(stat_jk[1][0])+","+str(stat_jk[1][1])+" in, " + \
          str(stat_jk[2][0])+","+str(stat_jk[2][1])+" out"
    print "FW stats: "+str(stat_fw)

    return passwd,insns,tss,ivt,hdl,khdl,(x_cr_key,x_cr_key_val),(a_std,a_xor,a_k),cg_divisor,r_deltas,0
#endef

# print out the TSS bitmaps
def write_tss(name, tss):
    fd = open( name, "w" )

    fd.write( "#include <tss.h>\n\n" )
    fd.write( "void tss_init(task_state_segment_t *tss)\n{\n" )
    for i in range(0,len(tss[0])):
        if tss[0][i] != 0xff:
            fd.write( "   tss->int_bitmap["+str(i)+"] = "+hex(tss[0][i])+";\n" )

    for i in range(0,len(tss[1])):
        if tss[1][i] != 0xff:
            fd.write( "   tss->io_bitmap["+str(i)+"] = "+hex(tss[1][i])+";\n" )

    fd.write("}\n")
    fd.close()
#endef

# print out user program
def write_user(name, ivt, hdl, main):
    fd = open( name, "w" )
    fd.write( ".section .user, \"ax\", @progbits\n.code16\n" )
    fd.write( "ivt:\n" )
    for e in ivt:
        fd.write( ivt[e]+"\n" )

    fd.write( "/*\n** IVT handlers\n*/\n" )

    # prevent compilation linearity :)
    random.shuffle(hdl)
    for e in hdl:
        if len(hdl[e]) != 0 :
            fd.write( hdl[e]+"\n" )

    fd.write( "\n.section .user.entry, \"ax\", @progbits\n.code16\n" )
    fd.write( "user_entry:\n" )
    for insn in main:
        fd.write( str_insn(insn)+"\n" )

    fd.close()
#endef

# print out kernel handlers
def write_krnl(path_src, path_inc, name, khdl):
    global kernel_start_rand
    global kernel_end_rand

    # handlers headers
    fd = open( path_inc+name+".h", "w" )
    fd.write( "#ifndef __V86_HDL_H__\n" + \
              "#define __V86_HDL_H__\n\n" + \
              "#include <types.h>\n\n" + \
              "/*\n" + \
              "** Functions\n" + \
              "*/\n\n" )

    for h in khdl:
        fd.write( h[1].split('\n')[0]+";\n" )

    fd.write( "void hdl_outb_1638(ctx_t *ctx);\n" )
    fd.write( "#endif\n" )
    fd.close()

    # worst complexity ever ;) O(1/0+)
    # generate random addrs in kernel binary range
    # then put handlers addrs
    # then write everything
    hdl = []
    for i in range(0,65536+256):
        hdl.append( random.randint(kernel_start_rand,kernel_end_rand) )

    for role in khdl:
        idx = role[2][1]
        if role[2][0] != 0:
            idx += 256
        hdl[idx] = role[1].split('(')[0].split(' ')[1]

    # add special 0x666
    hdl[0x666+256] = "hdl_outb_1638"

    fd = open( path_src+name+"_table.c", "w" )

    fd.write( "#include <v86_hdl.h>\n#include <interrupts.h>\n\n" )
    fd.write( "idt_handler_t hdl_table[] = {\n" )
    for h in hdl:
        fd.write( "   (idt_handler_t)"+str(h)+",\n" )

    fd.write( " };\n\n" )
    fd.close()

    # generate handlers file
    fd = open( path_src+name+".c", "w" )
    fd.write( "#include <v86_hdl.h>\n#include <klibc.h>\n\n" )

    # prevent compilation linearity :)
    random.shuffle( khdl )

    for role in khdl:
        fd.write( role[0]+"\n"+role[1]+"\n" )
    fd.close()

#endef

# print out kernel handlers
def write_excp(name, (xor_key,xor_val) ):

    fd = open( name, "w" )

    fd.write("""#include <exceptions.h>
#include <segmentation.h>
#include <interrupts.h>
#include <asmutils.h>
#include <map.h>
#include <klibc.h>

extern char __kernel_end__;
extern char __user_entry__;
extern char __user_start__;
extern char __user_end__;

""" )

    fd.write( "void exceptions_init(mem_map_t *map)\n{\n" )
    for e in range(0,32):
        excp = "   interrupts_set_isr( map->idt, "+str(e)+", "
        if e == 13 :
            excp += "general_protection_isr"
        else:
            excp += "(idt_handler_t)"+hex(random.randint(kernel_start_rand,kernel_end_rand))
        fd.write( excp+" );\n" )

    fd.write( """}

void general_protection_isr(ctx_t *k_ctx)
{
   mem_map_t            *map;
   task_state_segment_t *tss;
   ctx_t                *ctx;
   uint32_t              eip;
   uint8_t              *dst;

   map = (mem_map_t*)&__kernel_end__;
   tss = &map->tss;

   /* init user context */
   ctx = (ctx_t*)tss->esp0 - 1;
   memset( (void*)ctx, 0, sizeof(ctx_t) );

   ctx->cpu.ds = 0x60000>>4;
   ctx->cpu.es = 0x70000>>4;
   ctx->cpu.fs = 0x80000>>4;
   ctx->cpu.gs = 0x90000>>4;

   ctx->cpu.ss3    = 0x50000>>4;
   ctx->cpu.eflags = EFLAGS_VM|EFLAGS_IF;

   eip = (uint32_t)&__user_entry__ - (uint32_t)&__user_start__;
   ctx->cpu.cs     = (eip & 0xffff0000)>>4;
   ctx->cpu.eip    = (eip & 0xffff);

   ctx->gpr.esp = (uint32_t)&ctx->cpu.nr;

   ctx->sgr.ss = K_DATA_SELECTOR;
   ctx->sgr.ds = K_DATA_SELECTOR;
   ctx->sgr.gs = ctx->sgr.fs = ctx->sgr.es = ctx->sgr.ds;

   /* install pass in userland */
   dst = (uint8_t*)(ctx->cpu.ds<<4);
   memcpy( (void*)dst, (void*)tss->eax, tss->ecx );

   ctx->gpr.ecx = *(uint32_t*)(dst+12);

   /* install key in userland */
   dst = (uint8_t*)(ctx->cpu.fs<<4);

""" )

    fd.write( "   *((uint32_t*)dst  )  = 0x%x" % xor_key[0]+";\n" + \
              "   *((uint32_t*)dst+1)  = 0x%x" % xor_key[1]+";\n" + \
              "   *((uint32_t*)dst+2)  = 0x%x" % xor_key[2]+";\n" + \
              "   *((uint32_t*)dst+3)  = 0x%x" % xor_key[3]+";\n\n" + \
              "   *((uint32_t*)dst  ) ^= 0x%x" % xor_val+";\n" + \
              "   *((uint32_t*)dst+1) ^= 0x%x" % xor_val+";\n" + \
              "   *((uint32_t*)dst+2) ^= 0x%x" % xor_val+";\n" + \
              "   *((uint32_t*)dst+3) ^= 0x%x" % xor_val+";\n"
            )
    fd.write( """

   /* return to start_v86() bypassing fault */
   k_ctx->cpu.eip += 3;
}
""" )

    fd.close()
#endef

#
# Generate ascii art output
# and embed key
#
def encode_ascii_art(art,key):

    mod = len(art)%4
    if mod :
        art += " "*(4-mod)

    enc_art = []
    for i in range(0,len(art),4):
        enc_art.append( struct.unpack("<L", art[i:i+4])[0]^key )

    return enc_art,len(enc_art)
#endef

def write_answ( name, (a_std,a_xor,a_key) ):

    fail = "  __       _ _   _ \n" + \
           " / _| __ _(_) | | |\n" + \
           "| |_ / _` | | | | |\n" + \
           "|  _| (_| | | | |_|\n" + \
           "|_|  \__,_|_|_| (_)\n"

    win  = "                   _ _            _     _ \n" + \
           "  _____  _____ ___| | | ___ _ __ | |_  | |\n" + \
           " / _ \ \/ / __/ _ \ | |/ _ \ '_ \| __| | |\n" + \
           "|  __/>  < (_|  __/ | |  __/ | | | |_  |_|\n" + \
           " \___/_/\_\___\___|_|_|\___|_| |_|\__| (_)\n"

    enc_w,len_w = encode_ascii_art( win, a_key )
    enc_f,len_f = encode_ascii_art( fail, a_key )


    fd = open( name, "w" )
    fd.write( "#include <video.h>\n#include <print.h>\n\n" )

    fd.write( "static uint32_t xor_out[] = { " )
    for x in a_xor:
        fd.write( "0x%x, " % x )
    fd.write( "};\n" )

    fd.write( "static uint32_t asciiwin[] = { " )
    for x in enc_w:
        fd.write( "0x%x, " % x )
    fd.write( "};\n" )

    fd.write( "static uint32_t asciifail[] = { " )
    for x in enc_f:
        fd.write( "0x%x, " % x )
    fd.write( "};\n" )

    fd.write( """
void show(uint32_t *str, uint8_t l, uint8_t c, uint32_t len)
{
   uint32_t  i;
   for( i=0 ; i<len ; i++)
      str[i] ^= 0x%x;

   video_write_at( (char*)str, &c, &l, 1, 1 );

   for( i=0 ; i<len ; i++)
      str[i] ^= 0x%x;
}

void print_out(ctx_t *ctx)
{
   uint8_t   i;
   uint8_t   win;
   no_cast_t out;

   out.linear = (uint32_t)(ctx->cpu.es<<4)+(ctx->gpr.edi&0xffff);

   for( win=1, i=0 ; i<12 ; i++ )
      if( xor_out[i] != (out.u32[i]^0x%x) )
      {
         win=0;
         break;
      }

   if( win )
      show( asciiwin,
            video_get_line() + 3,
            VIDEO_DEFAULT_NB_COLUMNS/2 - 20 - 1,
            %d );
   else
      show( asciifail,
            video_get_line() + 3,
            VIDEO_DEFAULT_NB_COLUMNS/2 - 10 - 1,
            %d );
}
"""  % (a_key,a_key,a_key,len_w,len_f) )

    fd.close()
#endef



#
# - Wikipedia implementation
#   of Euclide extended algorithm
#
# - Way to find u,v in bezou theorem
#   u.a + v.b = pgcd(a,b)
#
def euclide(a,b):
    r  = a
    rp = b
    u  = 1
    v  = 0
    up = 0
    vp = 1

    q = 0
    rs = 0
    us = 0
    vs = 0

    while rp != 0:
        q = r/rp
        rs = r
        us = u
        vs = v
        r = rp
        u = up
        v = vp
        rp = rs - q*rp
        up = us - q*up
        vp = vs - q*vp

    return (r,u,v)
#endef

#
# Resolve a congruence system
# with Chinese Rest Theorem
#
def chinese_rest( ((a1,n1), (a2,n2), (a3,n3)) ):

    N = n1*n2*n3

    nc1 = N/n1
    nc2 = N/n2
    nc3 = N/n3

    r1,u1,v1 = euclide(n1,nc1)
    r2,u2,v2 = euclide(n2,nc2)
    r3,u3,v3 = euclide(n3,nc3)

    x = (a1*v1*nc1 + a2*v2*nc2 + a3*v3*nc3)

    return x%N
#endef

#
# Decryption routine for
# my XXTea implementation
#
def decrypt_sys(v,k,d):

    y = v[0]

    for sm in d:
        e = (sm>>2)&3
        p = 3
        while p > 0:
            z = v[p-1]
            v[p] = (v[p] - cr_mx(z,y,sm,p,k,e))&0xffffffff
            y = v[p]
            p -= 1

        z = v[3]
        v[0] = (v[0] - cr_mx(z,y,sm,p,k,e))&0xffffffff
        y = v[0]

    return struct.pack("<LLLL", v[0], v[1], v[2], v[3])
#endef

#
# Keygen to verify challenge
#
def keygen( result, xor_out, xor_out_v, divisor, xor_k, xor_k_v, rdeltas ):

    # xor embeded output
    clear_out = [ (x^xor_out_v)&0xffffffff for x in xor_out ]

    # split longs in shorts
    modulus = []
    for x in clear_out:
        modulus.append( x&0xffff )
        modulus.append( (x>>16)&0xffff )

    # ~ ror modulus
    for i in range(0,len(modulus)):
        nm = ~modulus[i]
        modulus[i] = ( ((nm&0xff)<<8) | ((nm>>8)&0xff) )&0xffff

    # rebuild congruence system with embedded divisor
    sys = []
    for i in range(0,8):
        sys.append( ((modulus[i], divisor[i][0]),
                    (modulus[i+8], divisor[i][1]),
                    (modulus[i+16], divisor[i][2])) )

    # resolve congruence system
    sol = []
    for i in range(0,8):
        sol.append( chinese_rest(sys[i]) )

    # build longs from shorts
    cipher = []
    for i in range(0,len(sol),2):
        cipher.append( (sol[i+1]<<16 | sol[i])&0xffffffff )

    # xor embeded key
    clear_key = [ (x^xor_k_v)&0xffffffff for x in xor_k ]

    # decrypt cipher
    passphrase = decrypt_sys(cipher,clear_key,rdeltas)

    if result != passphrase:
        print "\n!-!-!-!-!-! Keygen FAILURE !-!-!-!-!"
        sys.exit(1)
    else:
        print "\nKeygen success: "+passphrase
        print "\nAnswer e-mail : "+md5.md5(passphrase).hexdigest()+"@sstic.org\n"
#endef


#
# main
#
restart = 1
while restart:
    passphrase,main,tss,ivt,hdl,khdl,xk,ans,divisor,r_deltas,restart = gen()

pwd = os.getcwd()
path_src = pwd+"/kernel/src/gen/"
path_core = pwd+"/kernel/src/core/"
path_inc = pwd+"/kernel/include/"

write_excp( path_src+"exceptions.c", xk )
write_tss( path_src+"tss.c", tss )
write_krnl( path_src, path_inc,"v86_hdl", khdl )
write_user( path_src+"user.S", ivt, hdl, main )
write_answ( path_core+"output.c", ans )

keygen( passphrase, ans[1], ans[2], divisor, xk[0], xk[1], r_deltas )

if FAST:
    print "\n!!!!!!!!! WARNING, fast code generation !!!!!!!!\n"
